from oem_format_minimize.protocol.index import IndexMinimizeProtocol
from oem_format_minimize.protocol.metadata import MetadataMinimizeProtocol

from oem_format_minimize.protocol.show import ShowMinimizeProtocol
from oem_format_minimize.protocol.season import SeasonMinimizeProtocol
from oem_format_minimize.protocol.episode import EpisodeMinimizeProtocol

from oem_format_minimize.protocol.movie import MovieMinimizeProtocol
